#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import copy
import gzip
import os
import shutil
import sys
import tempfile
from multiprocessing import cpu_count

import get_vina
import requests
import yaml
from rdkit import Chem

from moldrug import fitness, home, utils
from moldrug.data import get_data

# This is in case vina is not found. You can change the vina executable here
vina_executable = os.path.abspath('vina')

if not os.path.isfile(vina_executable):
    get_vina.download()


# Creating a temporal directory
tmp_path = tempfile.TemporaryDirectory()
wd = tmp_path.name
os.chdir(wd)

# Creating receptors files
TEST_DATA = {
    'x0161': get_data('x0161'),
    '6lu7': get_data('6lu7')
}

# Getting the crem data base
url = "http://www.qsar4u.com/files/cremdb/replacements02_sc2.db.gz"
r = requests.get(url, allow_redirects=True)
crem_dbgz_path = 'crem.db.gz'
crem_db_path = 'crem.db'
open(crem_dbgz_path, 'wb').write(r.content)
with gzip.open(crem_dbgz_path, 'rb') as f_in:
    with open(crem_db_path, 'wb') as f_out:
        shutil.copyfileobj(f_in, f_out)


def test_single_receptor_command_line():
    Config = {
        "01_grow": {
            "type": "GA",
            "njobs": 3,
            "seed_mol": TEST_DATA['x0161']['smiles'],
            "AddHs": True,
            "costfunc": "Cost",
            "costfunc_kwargs": {
                "vina_executable": vina_executable,
                "receptor_pdbqt_path": TEST_DATA['x0161']['protein']['pdbqt'],
                "boxcenter": TEST_DATA['x0161']['box']['boxcenter'],
                "boxsize": TEST_DATA['x0161']['box']['boxsize'],
                "exhaustiveness": 4,
                "ncores": 4,
                "num_modes": 1
            },
            "crem_db_path": crem_db_path,
            "maxiter": 1,
            "popsize": 2,
            "beta": 0.001,
            "pc": 1,
            "get_similar": False,
            "mutate_crem_kwargs": {
                "radius": 3,
                "min_size": 0,
                "max_size": 0,
                "min_inc": -5,
                "max_inc": 6,
                "ncores": 1
            },
            "save_pop_every_gen": 10,
            "deffnm": "01_grow"
        },
        "02_allow_grow": {
            "mutate_crem_kwargs": {
                "radius": 3,
                "min_size": 1,
                "max_size": 0,
                "min_inc": -5,
                "max_inc": 3,
                "ncores": 12
            },
            "maxiter": 1,
            "deffnm": "02_allow_grow"
        }
    }
    with open("test_single_receptor.yml", 'w') as c:
        yaml.dump(Config, c)
    p = utils.run('moldrug test_single_receptor.yml')
    print(p.stdout)
    # Run a second time but with a seed population
    Config['01_grow']['seed_mol'] = ['02_allow_grow_pop.pbz2', '02_allow_grow_pop.pbz2']
    with open("test_single_receptor_init_pop.yml", 'w') as c:
        yaml.dump(Config, c)
    p = utils.run('moldrug test_single_receptor_init_pop.yml')
    print(p.stdout)


def test_multi_receptor(maxiter=1, popsize=2, njobs=3, NumbCalls=1):
    out = utils.GA(
        seed_mol=[Chem.MolFromSmiles(TEST_DATA['x0161']['smiles']), Chem.MolFromSmiles(TEST_DATA['x0161']['smiles'])],
        AddHs=True,
        maxiter=maxiter,
        popsize=popsize,
        crem_db_path=crem_db_path,
        pc=1,
        get_similar=True,
        mutate_crem_kwargs={
            'radius': 3,
            'min_size': 0,
            'min_inc': -5,
            'max_inc': 3,
            'replace_ids': [3, 4, 5, 7],
            'protected_ids': [0],
        },
        costfunc=fitness.CostMultiReceptors,
        costfunc_kwargs={
            'receptor_pdbqt_path': [TEST_DATA['x0161']['protein']['pdbqt'], TEST_DATA['6lu7']['protein']['pdbqt']],
            'boxcenter': [TEST_DATA['x0161']['box']['boxcenter'], TEST_DATA['6lu7']['box']['boxcenter']],
            'boxsize': [TEST_DATA['x0161']['box']['boxsize'], TEST_DATA['6lu7']['box']['boxsize']],
            'vina_score_type': ['min', 'max'],
            'exhaustiveness': 4,
            'vina_executable': vina_executable,
            'ncores': int(cpu_count() / njobs),
            'num_modes': 1,
            'vina_seed': 1234,
        },
        save_pop_every_gen=20,
        deffnm='test_multi_receptor',
        randomseed=123)

    for _ in range(NumbCalls):
        out(njobs=njobs)

    for o in out.pop:
        print(o.smiles, o.cost)
    out.pickle(f"result_test_multi_receptor_NumGens_{out.NumGens}_PopSize_{popsize}", compress=False)
    print(out.to_dataframe())

    with open('vina_out.pdbqt', 'w') as p:
        p.write(out.pop[0].pdbqt[0])

    vina_out = utils.VINA_OUT('vina_out.pdbqt')
    vina_out.chunks[0].get_atoms()
    vina_out.chunks[0].write('chunk.pdbqt')
    vina_out.chunks[0].write()


def test_local_command_line():
    Config = {
        "main": {
            "type": "Local",
            "njobs": 1,
            "pick": 1,
            "AddHs": True,
            "seed_mol": Chem.MolToSmiles(Chem.AddHs(Chem.MolFromSmiles(TEST_DATA['x0161']['smiles']))),
            "costfunc": "CostOnlyVina",
            "costfunc_kwargs": {
                "vina_executable": vina_executable,
                "receptor_pdbqt_path": TEST_DATA['x0161']['protein']['pdbqt'],
                "boxcenter": TEST_DATA['x0161']['box']['boxcenter'],
                "boxsize": TEST_DATA['x0161']['box']['boxsize'],
                "exhaustiveness": 4,
                "num_modes": 1
            },
            "crem_db_path": crem_db_path,
            "grow_crem_kwargs": {
                "radius": 3,
                "min_atoms": 1,
                "max_atoms": 3
            }
        }
    }
    with open("local_config.yml", 'w') as c:
        yaml.dump(Config, c)

    utils.run(f"moldrug local_config.yml --fitness {os.path.join(home.home(), 'fitness.py')}")
    print(os.listdir())
    # This problems with the modules are not so convenient
    sys.path.append('.')
    result = utils.decompress_pickle('local_result.pbz2')
    result.pickle('local_non_compress', compress=False)
    print(result.to_dataframe())


# @pytest.mark.filterwarnings("ignore:\nVina failed")
def test_fitness_module():
    individual = utils.Individual(Chem.MolFromSmiles(TEST_DATA['x0161']['smiles']))
    individual_corrupted = copy.deepcopy(individual)
    individual_corrupted.pdbqt = 'This is a corrupted pdbqt'
    receptor_pdbqt_path = [TEST_DATA['x0161']['protein']['pdbqt'], TEST_DATA['6lu7']['protein']['pdbqt']]
    boxcenter = [TEST_DATA['x0161']['box']['boxcenter'], TEST_DATA['6lu7']['box']['boxcenter']]
    boxsize = [TEST_DATA['x0161']['box']['boxsize'], TEST_DATA['6lu7']['box']['boxsize']]
    vina_score_type = ['min', 'max']

    fitness.Cost(
        Individual=copy.deepcopy(individual),
        wd=wd,
        vina_executable=vina_executable,
        receptor_pdbqt_path=TEST_DATA['x0161']['protein']['pdbqt'],
        boxcenter=TEST_DATA['x0161']['box']['boxcenter'],
        boxsize=TEST_DATA['x0161']['box']['boxsize'],
        exhaustiveness=4,
        ncores=4)
    fitness.Cost(
        Individual=copy.deepcopy(individual),
        wd=wd,
        vina_executable=vina_executable,
        receptor_pdbqt_path=TEST_DATA['x0161']['protein']['pdbqt'],
        boxcenter=TEST_DATA['x0161']['box']['boxcenter'],
        boxsize=TEST_DATA['x0161']['box']['boxsize'],
        exhaustiveness=4,
        ncores=4,
        constraint=True,
        constraint_type='local_only',
        constraint_ref=Chem.MolFromMolFile(TEST_DATA['x0161']['ligand_3D']),
        constraint_receptor_pdb_path=TEST_DATA['x0161']['protein']['pdb'])

    fitness.Cost(Individual=copy.deepcopy(individual_corrupted), wd=wd,
                 receptor_pdbqt_path=TEST_DATA['x0161']['protein']['pdbqt'],
                 vina_executable=vina_executable,
                 boxcenter=TEST_DATA['x0161']['box']['boxcenter'], boxsize=TEST_DATA['x0161']['box']['boxsize'],
                 exhaustiveness=4, ncores=4, vina_seed=1234)
    fitness.CostMultiReceptors(
        Individual=copy.deepcopy(individual_corrupted), wd=wd, receptor_pdbqt_path=receptor_pdbqt_path,
        vina_executable=vina_executable, vina_score_type=vina_score_type, boxcenter=boxcenter,
        boxsize=boxsize, exhaustiveness=4, ncores=4, vina_seed=1234)
    fitness.CostMultiReceptorsOnlyVina(
        Individual=copy.deepcopy(individual), wd=wd, receptor_pdbqt_path=receptor_pdbqt_path,
        vina_executable=vina_executable, vina_score_type=vina_score_type, boxcenter=boxcenter,
        boxsize=boxsize, exhaustiveness=4, ncores=4, vina_seed=1234)
    fitness.CostMultiReceptorsOnlyVina(
        Individual=copy.deepcopy(individual), wd=wd, receptor_pdbqt_path=receptor_pdbqt_path,
        vina_executable=vina_executable, vina_score_type=vina_score_type, boxcenter=boxcenter,
        boxsize=boxsize, exhaustiveness=4, ncores=4, wt_cutoff=2, vina_seed=1234)
    fitness.CostMultiReceptorsOnlyVina(Individual=copy.deepcopy(
        individual_corrupted), wd=wd, receptor_pdbqt_path=receptor_pdbqt_path,
        vina_executable=vina_executable, vina_score_type=vina_score_type, boxcenter=boxcenter,
        boxsize=boxsize, exhaustiveness=4, ncores=4, vina_seed=1234)

    fitness.CostOnlyVina(
        Individual=copy.deepcopy(individual), wd=wd,
        receptor_pdbqt_path=TEST_DATA['x0161']['protein']['pdbqt'], vina_executable=vina_executable,
        boxcenter=TEST_DATA['x0161']['box']['boxcenter'], boxsize=TEST_DATA['x0161']['box']['boxsize'],
        exhaustiveness=4, ncores=4, vina_seed=1234)
    fitness.CostOnlyVina(
        Individual=copy.deepcopy(individual), wd=wd,
        receptor_pdbqt_path=TEST_DATA['x0161']['protein']['pdbqt'], vina_executable=vina_executable,
        boxcenter=TEST_DATA['x0161']['box']['boxcenter'], boxsize=TEST_DATA['x0161']['box']['boxsize'],
        exhaustiveness=4, ncores=4, wt_cutoff=2, vina_seed=1234)
    fitness.CostOnlyVina(
        Individual=copy.deepcopy(individual_corrupted), wd=wd,
        receptor_pdbqt_path=TEST_DATA['x0161']['protein']['pdbqt'], vina_executable=vina_executable,
        boxcenter=TEST_DATA['x0161']['box']['boxcenter'], boxsize=TEST_DATA['x0161']['box']['boxsize'],
        exhaustiveness=4, ncores=4, vina_seed=1234)

    fitness.__get_mol_cost(
        mol=Chem.MolFromMolFile(TEST_DATA['x0161']['ligand_3D']), wd=wd,
        receptor_pdbqt_path=TEST_DATA['x0161']['protein']['pdbqt'], vina_executable=vina_executable,
        boxcenter=TEST_DATA['x0161']['box']['boxcenter'], boxsize=TEST_DATA['x0161']['box']['boxsize'],)

    # Clean
    utils.tar_errors()
    os.remove('error.tar.gz')


def test_home():
    home.home(dataDir='data')


def test_get_sim_utils():
    from rdkit.Chem import AllChem
    mols = []
    ref_fps = []
    for s in ['CC', 'CCO']:
        mol = Chem.MolFromSmiles(s)
        mols.append(mol)
        ref_fps.append(AllChem.GetMorganFingerprintAsBitVect(mol, 2))
    utils.get_sim(mols, ref_fps)


def test_lipinski():
    mol = Chem.MolFromSmiles('CCCO')
    utils.lipinski_filter(Chem.MolFromSmiles('BrCC(COCN)CC(Br)CC(Cl)CCc1ccccc1CCCC(NCCCO)'))
    utils.lipinski_filter(mol)
    utils.lipinski_profile(mol)


def test_Individual():
    I1 = utils.Individual(Chem.MolFromSmiles('CC'), cost=10)
    I2 = utils.Individual(Chem.MolFromSmiles('CCO'), cost=2)
    I3 = copy.copy(I1)
    I4 = copy.deepcopy(I2)
    I5 = utils.Individual(Chem.MolFromSmiles('CC'), cost=10, pdbqt=I1.pdbqt)
    assert I3 + I4 == 12
    assert I5 - I2 == 8
    assert I1 > I2
    assert I1 >= I2
    assert I2 < I1
    assert I2 <= I1
    assert -I1 == -10
    assert abs(I1) == 10
    assert I1 * I2 == 20
    assert I1 / I2 == 5
    assert I1 // I2 == 5
    assert I1 % I2 == 0
    assert divmod(I1, I2) == (5, 0)
    assert I1**I2 == 100


def test_miscellanea():
    obj0 = []
    for i in range(0, 50):
        obj0.append(utils.NominalTheBest(Value=i, LowerLimit=10, Target=20, UpperLimit=30))

    utils.full_pickle('test_desirability', obj0)
    utils.compressed_pickle('test_desirability', obj0)
    obj1 = utils.loosen('test_desirability.pkl')
    obj2 = utils.decompress_pickle('test_desirability.pbz2')

    assert obj0 == obj1
    assert obj0 == obj2


def test_constraintconf():
    from moldrug.constraintconf import constraintconf
    with Chem.SDWriter('fix.sdf') as w:
        mol = Chem.MolFromMolFile(TEST_DATA['x0161']['ligand_3D'])
        w.write(mol)
    with open('mol.smi', 'w') as f:
        f.write(TEST_DATA['x0161']['smiles'])

    constraintconf(
        pdb=TEST_DATA['x0161']['protein']['pdb'],
        smi='mol.smi',
        fix='fix.sdf',
        out='conf.sdf',
        randomseed=1234,
    )
    # Clean
    utils.tar_errors()


def test_generate_conformers():
    from rdkit.Chem import AllChem

    from moldrug.constraintconf import generate_conformers
    ref = Chem.MolFromSmiles('O=S(=O)(Nc1ccc(Cl)cc1)c1ccsc1C(O)O')
    mol = Chem.MolFromSmiles('CN(C)S(=O)(=O)c1cc(NS(=O)(=O)c2ccsc2C(O)O)ccc1Cl')

    AllChem.EmbedMolecule(ref)
    AllChem.MMFFOptimizeMolecule(ref)
    generate_conformers(Chem.RemoveHs(mol), Chem.RemoveHs(ref), 50, randomseed=1234)

    # Clean
    utils.tar_errors()


if __name__ == '__main__':
    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
For information of MolDrug:
    Docs: https://moldrug.readthedocs.io/en/latest/
    Source Code: https://github.com/ale94mleon/moldrug
"""
import os
from moldrug._version import __version__

__author__ = "Alejandro Martínez León"
__email__ = "ale94mleon@gmail.com"

if "MOLDRUG_VERBOSE" in os.environ:
    if os.environ["MOLDRUG_VERBOSE"].lower() in [1, 'true']:
        verbose = True
    elif os.environ["MOLDRUG_VERBOSE"].lower() in [0, 'false']:
        verbose = False
    else:
        raise ValueError(f"MOLDRUG_VERBOSE = {os.environ['MOLDRUG_VERBOSE']} is invalid. Choose from: 1, true, false (case insensitive).")
else:
    verbose = False



import yaml
from moldrug import home
import os

_AVAILABLE_DATA = ['6lu7', 'x0161']


class DataNotFound(Exception):
    pass


def get_data(name: str) -> dict:
    """Retrieve MolDrug data

    Parameters
    ----------
    name : str
        Name of the data directory of MolDrug

    Returns
    -------
    dict
        A dictionary with keys:

        * box: boxcenter, boxsize
        * ligand_3D: absolute path
        * smiles: the ligans's SMILES
        * protein: pdb, pdbqt

    Raises
    ------
    DataNotFound
        MolDrug does not contains the data.
    """

    root_data = home.home(dataDir=name)

    if not os.path.exists(root_data) or not name:
        raise DataNotFound(f"MolDrug data does not have '{name}' data set. "
                           f"Choose from: {_AVAILABLE_DATA}")

    with open(os.path.join(root_data, 'box.yml'), 'r') as f:
        box = yaml.safe_load(f)

    with open(os.path.join(root_data, 'ligand.smi'), 'r') as f:
        smiles = f.readline()

    data = {
        'box': box,
        'ligand_3D': os.path.join(root_data, 'ligand_3D.mol'),
        'smiles': smiles,
        'protein': {
            'pdb': os.path.join(root_data, 'protein.pdb'),
            'pdbqt': os.path.join(root_data, 'protein.pdbqt'),
        }
    }
    return data



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
For information of MolDrug:
    Docs: https://moldrug.readthedocs.io/en/latest/
    Source Code: https://github.com/ale94mleon/moldrug
"""
from moldrug import utils, constraintconf, __version__
import yaml
import argparse
import inspect
import os
import sys
import datetime
from rdkit import Chem
from typing import Union


class CommandLineHelper:
    def __init__(self, parser) -> None:
        self.args = parser.parse_args()
        self.yaml_file = self.args.yaml_file
        self.fitness = self.args.fitness
        self.continuation = self.args.continuation
        self.verbose = self.args.verbose
        self._set_attributes()

        if self.verbose:
            os.environ['MOLDRUG_VERBOSE'] = 'True'

    def _set_attributes(self):
        # Get and set configuration
        self._set_config()
        # It will generate costfunc attribute
        self._set_costfunc()
        # it will generate the attribute TypeOfRun
        self._set_TypeOfRun()
        # it will generate the attributes: MainConfig, FollowConfig, InitArgs, CallArgs and MutableArgs
        self._translate_config()
        # Here FollowConfig is updated and the corresponded initialization occurs if self.continuation.
        # pbz2 and new_maxiter attributes are generated
        self._set_init_MolDrugClass()

    def _set_config(self):
        with open(self.yaml_file, 'r') as c:
            self.configuration = yaml.safe_load(c)

    def _split_config(self):
        config = self.configuration.copy()
        MainConfig = config.pop(list(config.keys())[0])
        FollowConfig = config
        return MainConfig, FollowConfig

    def _set_costfunc(self):
        if self.fitness:
            # If the fitness module provided is not in the current directory or if its name is not fitness
            with open(self.fitness, 'r') as source:
                with open('CustomMolDrugFitness.py', 'w') as destination:
                    destination.write(source.read())
            sys.path.append('.')
            import CustomMolDrugFitness
            costfunc = dict(inspect.getmembers(CustomMolDrugFitness))[self._split_config()[0]['costfunc']]
        else:
            from moldrug import fitness
            costfunc = dict(inspect.getmembers(fitness))[self._split_config()[0]['costfunc']]
        self.costfunc = costfunc

    def _set_TypeOfRun(self):
        self._TypeOfRun_str = self._split_config()[0]['type'].lower()
        if self._TypeOfRun_str == 'ga':
            self.TypeOfRun = utils.GA
        elif self._TypeOfRun_str == 'local':
            self.TypeOfRun = utils.Local
        else:
            raise NotImplementedError(f"\"{self._split_config()[0]['type']}\" it is not a possible type. "
                                      "Select from: GA or Local")

    def _translate_config(self):
        MainConfig, FollowConfig = self._split_config()

        # Convert the SMILES (or path to compress_pickle) to RDKit mol (or list of RDkit mol)
        if self._TypeOfRun_str == 'local':
            MainConfig['seed_mol'] = Chem.MolFromSmiles(MainConfig['seed_mol'])
        else:
            # TODO make clear errors in case the path does not exist or it was not possible to create a molecule
            if isinstance(MainConfig['seed_mol'], list):
                # If the items are path to the pickle objects
                if any([os.path.isfile(path) for path in MainConfig['seed_mol']]):
                    seed_pop = set()
                    for solution in MainConfig['seed_mol']:
                        _, pop = utils.decompress_pickle(solution)
                        seed_pop.update(pop)
                    # Sort
                    seed_pop = sorted(seed_pop)
                    # Select the best and get only the RDKit molecule object
                    MainConfig['seed_mol'] = [individual.mol for individual in seed_pop[:MainConfig['popsize']]]
                else:
                    # Delete repeated SMILES
                    MainConfig['seed_mol'] = set(MainConfig['seed_mol'])
                    # COnvert to mol
                    MainConfig['seed_mol'] = [Chem.MolFromSmiles(smi) for smi in MainConfig['seed_mol']]
                    # Filter out invalid molecules
                    MainConfig['seed_mol'] = list(filter(None, MainConfig['seed_mol']))
            else:  # It will be assumed that it is a valid SMILES string
                MainConfig['seed_mol'] = Chem.MolFromSmiles(MainConfig['seed_mol'])

        # Convert if needed constraint_ref
        if 'constraint_ref' in MainConfig['costfunc_kwargs']:
            MainConfig['costfunc_kwargs']['constraint_ref'] = Chem.MolFromMolFile(
                MainConfig['costfunc_kwargs']['constraint_ref'])

        InitArgs = MainConfig.copy()

        # Modifying InitArgs
        _ = [InitArgs.pop(key, None) for key in ['type', 'njobs', 'pick']]
        InitArgs['costfunc'] = self.costfunc

        # Getting call arguments
        CallArgs = dict()
        for key in ['njobs', 'pick']:
            try:
                CallArgs[key] = MainConfig[key]
            except KeyError:
                pass

        # Checking for follow jobs and sanity check on the arguments
        if FollowConfig:
            # Defining the possible mutable arguments with its default values depending on the type of run
            if MainConfig['type'].lower() == 'local':
                raise ValueError("Type = Local does not accept multiple call from the command line! Remove follow "
                                 "jobs from the yaml file (only the main job is possible)")
            else:
                # Add default value in case it is not provided for keyword arguments
                list_of_keywords = [
                    'beta', 'pc', 'get_similar', 'mutate_crem_kwargs',
                    'save_pop_every_gen', 'checkpoint', 'deffnm',
                ]
                for param in inspect.signature(self.TypeOfRun).parameters.values():
                    if (param.kind == param.POSITIONAL_OR_KEYWORD and
                            param.default is not param.empty and
                            param.name in list_of_keywords and
                            param.name not in InitArgs):
                        InitArgs[param.name] = param.default

                MutableArgs = {
                    'njobs': CallArgs['njobs'],
                    'crem_db_path': InitArgs['crem_db_path'],
                    'maxiter': InitArgs['maxiter'],
                    'popsize': InitArgs['popsize'],
                    'beta': InitArgs['beta'],
                    'pc': InitArgs['pc'],
                    'get_similar': InitArgs['get_similar'],
                    # This one it will update with the default values of crem rather thant the previous one.
                    'mutate_crem_kwargs': InitArgs['mutate_crem_kwargs'],
                    'save_pop_every_gen': InitArgs['save_pop_every_gen'],
                    'checkpoint': InitArgs['checkpoint'],
                    'deffnm': InitArgs['deffnm'],
                }

            # Sanity check
            for job in FollowConfig:
                for arg in FollowConfig[job]:
                    if arg not in MutableArgs:
                        raise ValueError(f"The job: {job} has a non-valid argument \"{arg}\". "
                                         f"For now only the following are accepted: {list(MutableArgs.keys())}")
        else:
            MutableArgs = None

        self.MainConfig = MainConfig
        self.FollowConfig = FollowConfig
        self.InitArgs = InitArgs
        self.CallArgs = CallArgs
        self.MutableArgs = MutableArgs

    def _get_continuation_point(self):
        """
        This gave me the job and how many generation are needed to complete it.
        The further jobs are suppose that must run.
        """
        if self.continuation:
            if self._TypeOfRun_str != 'ga':
                raise RuntimeError('Continuation is only valid for GA runs.')
            # Check what was already done
            total_iter = 0
            pbz2 = None
            for job in self.configuration:
                # El problema es que no estan los archivos entonces hay que modificar total_iter
                if os.path.isfile(f"{self.configuration[job]['deffnm']}_result.pbz2"):
                    # must be defined maxiter in the configuration file
                    total_iter += self.configuration[job]['maxiter']
                    # Delete (update) the jobs in self.FollowConfig if they were already done
                    if job in self.FollowConfig:
                        del self.FollowConfig[job]
                    # Stay with the last one
                    pbz2 = f"{self.configuration[job]['deffnm']}_result.pbz2"

            # If there is a continuation file, use this
            if os.path.isfile("cpt.pbz2"):
                pbz2 = 'cpt.pbz2'
                iter_done = utils.decompress_pickle(pbz2).NumGens
                total_iter = 0
                for job in self.configuration:
                    total_iter += self.configuration[job]['maxiter']
                    if total_iter >= iter_done:
                        del self.FollowConfig[job]
                        break
            elif pbz2:
                iter_done = utils.decompress_pickle(pbz2).NumGens
            else:
                iter_done = 0
            new_maxiter = total_iter - iter_done
        else:
            # In this case we must start from scratch. There are not .pbz2 files in the directory
            pbz2, new_maxiter = None, 0

        # Set the attributes
        self.pbz2 = pbz2
        self.new_maxiter = new_maxiter

    def _set_init_MolDrugClass(self):
        # Here is where the continuation code is added

        # Get if if needed to continue and make the corresponded updates on self.FollowConfig
        self._get_continuation_point()

        if self.pbz2:
            self.MolDrugClass = utils.decompress_pickle(self.pbz2)
            self.MolDrugClass.maxiter = self.new_maxiter
        else:
            # Initialize the class from scratch
            self.MolDrugClass = self.TypeOfRun(**self.InitArgs)

    def run_MolDrugClass(self):
        self.MolDrugClass(**self.CallArgs)

    def save_data(self):
        # Saving data
        if self._TypeOfRun_str == 'local':
            self.MolDrugClass.pickle("local_result", compress=True)
            utils.make_sdf(self.MolDrugClass.pop, sdf_name="local_pop")
        else:
            self.MolDrugClass.pickle(f"{self.MolDrugClass.deffnm}_result", compress=True)
            utils.make_sdf(self.MolDrugClass.pop, sdf_name=f"{self.MolDrugClass.deffnm}_pop")

    def __repr__(self) -> str:
        string = self.args.__repr__().replace('Namespace', self.__class__.__name__)
        if self.continuation:
            string += f"\nContinuationPoint(pbz2={self.pbz2}, do_iter={self.new_maxiter})"
        return string


def __moldrug_cmd():
    """
    This function is only used in as part of the command line interface of MolDrug.
    It makes possible to use MolDrug form the command line. More detail help is available
    from the command line `moldrug -h`.

    Raises
    ------
    NotImplementedError
        In case that the type of the calculation differs from Local or GA (currently implementations)
    ValueError
        In case that the user ask for followed jobs and Local is selected.
    ValueError
        In case that a non-mutable or non-defined argument is given by the user on the follow jobs.
    """
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument(
        help='The configuration yaml file',
        dest='yaml_file',
        type=str)
    parser.add_argument("-f", "--fitness",
                        help="The path to the user-custom fitness module; inside of which the given custom "
                        "cost function must be implemented. "
                        "See the docs for how to do it properly. E.g. my/awesome/fitness_module.py. "
                        "By default will look in the moldrug.fitness module.",
                        dest="fitness",
                        nargs=argparse.OPTIONAL,
                        default=None,
                        type=str)
    parser.add_argument("-c", "--continue",
                        help="To continue the simulation. The MolDrug command must be the same "
                        "and all the output MolDrug files must be located "
                        "in the working directory. This option is only compatible "
                        "with moldrug.utils.GA; otherwise, a RuntimeError will be raised.",
                        action="store_true",
                        dest="continuation")
    parser.add_argument(
        '-v', '--version',
        action='version',
        version=f"moldrug: {__version__}")
    parser.add_argument(
        '-V', '--verbose',
        nargs="?",
        dest='verbose',
        const=True,
        default=False,
        type=bool
    )
    UserArgs = CommandLineHelper(parser)

    print(
        f"Started at {datetime.datetime.now().strftime('%c')}\n"
        f"You are using moldrug: {__version__}.\n\n"
        f"{UserArgs}\n\n")

    # Call the class
    UserArgs.run_MolDrugClass()
    # Saving data
    UserArgs.save_data()
    # print('The main job finished!')

    # In case that follows jobs were defined
    if UserArgs.FollowConfig:
        MutableArgs = UserArgs.MutableArgs.copy()
        for job in UserArgs.FollowConfig:
            print(f"The follow job {job} started.")

            # Updating arguments
            MutableArgs.update(UserArgs.FollowConfig[job])
            InitArgs = MutableArgs.copy()

            # Changing the attributes values
            for arg in InitArgs:
                setattr(UserArgs.MolDrugClass, arg, InitArgs[arg])

            # Call the class again
            UserArgs.run_MolDrugClass()
            # Saving data
            UserArgs.save_data()
            print(f'The job {job} finished!')

    # Clean checkpoint on normal end
    if os.path.isfile('cpt.pbz2'):
        os.remove('cpt.pbz2')


def __constraintconf_cmd():
    """
    Command line implementation for :meth:`moldrug.constraintconf.constraintconf`
    """
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument(
        "--pdb",
        help="Protein pdb file",
        dest="pdb",
        type=str,
    )
    parser.add_argument(
        "--smi",
        help="Input SMILES file name",
        dest="smi",
        type=str,
    )
    parser.add_argument(
        "--fix",
        help="File with fixed piece of the molecule",
        dest="fix",
        type=str,
    )
    parser.add_argument(
        "--out",
        help="Output file name",
        dest="out",
        type=str,
    )
    parser.add_argument(
        "--max",
        help="Maximum number of conformers to generate, by default %(default)s",
        dest="max",
        default=25,
        type=int,
    )
    parser.add_argument(
        "--rms",
        help="RMS cutoff, by default %(default)s",
        dest="rms",
        default=0.01,
        type=float,
    )
    parser.add_argument(
        "--bump",
        help="Bump cutoff, by default %(default)s",
        dest="bump",
        default=1.5,
        type=float,
    )
    parser.add_argument(
        "--seed",
        help="Provide a seed for the random number generator so that "
        "the same coordinates can be obtained for a molecule on multiple runs. "
        "If None, the RNG will not be seeded, by default None %(default)s",
        dest="seed",
        default=None,
        type=Union[int, None],
    )
    args = parser.parse_args()
    constraintconf.constraintconf(
        pdb=args.pdb,
        smi=args.smi,
        fix=args.fix,
        out=args.out,
        max_conf=args.max,
        rms=args.rms,
        bump=args.bump,
        randomseed=args.seed)


if __name__ == '__main__':
    pass
